;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x29ba45b3d588ba3ddf69c1cc01110fd1 . "./code.cg2js/cg-acache.fasl")
(#xccd047450e616f2947a9e97b847ef693 . "./code.cg2js/cg-base.fasl")
(#x8f0f96362a832f45792cfcd6c7966f4e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x0b7aa1d97396a5e772d382ef4c3a0d0d . "./code.cg2js/cg-bitmap-pane.fasl")
(#x702d6c1fa8e025d5e5b5780d24009fec . "./code.cg2js/cg-bitmap-stream.fasl")
(#x7bde7b0c458c9bbba53994b0d085d042 . "./code.cg2js/cg-button.fasl")
(#xf8a8d0fd331ee97f2af3922aab9022bf . "./code.cg2js/cg-calendar.fasl")
(#xf7a512a6d88d8f5a4a4297e20bac2ee8 . "./code.cg2js/cg-caret.fasl")
(#x9a5e244fc3471b6580471264a50f1455 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x4af0ccc12b2645cbaee06d0401ad1a8f . "./code.cg2js/cg-chart-widget.fasl")
(#x9d4c6d1ab5e469409447f4be7441db9f . "./code.cg2js/cg-check-box.fasl")
(#x0cbb476c0d7dae31f076271acfd161cd . "./code.cg2js/cg-choice-list.fasl")
(#x2d4ec21086def07a17db16924666ab55 . "./code.cg2js/cg-choose-printer.fasl")
(#x55d37c3ef6f2ec29ec3c1168376484ee . "./code.cg2js/cg-class-grid.fasl")
(#xd5c603c4588c4bef76f2c1bd170c6219 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x0fa64c5bd0fafc8d2d2483cfb64f47b5 . "./code.cg2js/cg-class-support.fasl")
(#x9e11be0e569448e97aeaa38e9018113d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x15848a10f72b198902f411c1995b1fc7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x52465fb74b4d8435b275c3ac8272220d . "./code.cg2js/cg-clipboard.fasl")
(#x3e9ebc5e96259361ac3bfac5c659f968 . "./code.cg2js/cg-color-dialog.fasl")
(#x552cea0466dc7f7518dd71cc976995d9 . "./code.cg2js/cg-combo-box.fasl")
(#x7a86a06f15ffb43136a67a9b7e830889 . "./code.cg2js/cg-common-control.fasl")
(#x6889ba565a0a5272e900ec9431289601 . "./code.cg2js/cg-comtab.fasl")
(#x7c9f5998767078967cfd2f9e8368c4f0 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xa7daf0c98289c4158e7f702d54a16008 . "./code.cg2js/cg-curve.fasl")
(#xd09babf4925fb0d196c2095c7718c780 . "./code.cg2js/cg-dde-utils.fasl")
(#x4068d77bf3fcdbff99b9c55efa4f44be . "./code.cg2js/cg-dev.fasl")
(#x442a338e803ee1f0d520142f6ff1701c . "./code.cg2js/cg-dialog-item.fasl")
(#x5c844d4923476f815a0e4832d1639ef0 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x9497d7345a1b4eb7dc03d5fdc213e583 . "./code.cg2js/cg-directory-dialog.fasl")
(#x0e64498d6307e561abcad44ab987f08f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x7d0534cc65ba07f0956d3a45b1774314 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x7e351eaf112f9277c2cd43a3b8e73bf3 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xddafce09b3084da48712f1be31e9dac3 . "./code.cg2js/cg-drawable.fasl")
(#xed573d575c9636397ad5ebe8217fa46e . "./code.cg2js/cg-dropping-outline.fasl")
(#x07efd10c57628b4111cf948e93df3188 . "./code.cg2js/cg-edit-in-place.fasl")
(#x4f67f7a9a3fb6c34fd671db7e64db6cf . "./code.cg2js/cg-editable-text.fasl")
(#x8f396683f05b81ac971ef97f803610cc . "./code.cg2js/cg-file-dialog.fasl")
(#x8aef72e89843b3a5ed45b3c3fc77b578 . "./code.cg2js/cg-fill-texture.fasl")
(#xc07940b5219a314f1be512b97052125c . "./code.cg2js/cg-find-string-dialog.fasl")
(#x3a9dcc5230bda11fdfdf77a2c84b45ca . "./code.cg2js/cg-font-dialog.fasl")
(#xead26e0ca4091d88f70729e417159abc . "./code.cg2js/cg-gesture-emulation.fasl")
(#xd5d47f67c53d908504ae0ef35a00df76 . "./code.cg2js/cg-get-pixmap.fasl")
(#xa93ea598b02c09853cc4d01168dc24a2 . "./code.cg2js/cg-get-position.fasl")
(#xd20329aa16526b7df31659a79bdd5c90 . "./code.cg2js/cg-graphics-context.fasl")
(#x67c679ad27b8944bc75071f3ffe56196 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x2d93e4d95447ac3529685d690de48e13 . "./code.cg2js/cg-grid-widget.fasl")
(#xf493f106195febe9c66779ddb3a7c9d9 . "./code.cg2js/cg-group-box.fasl")
(#xa7d3e89a6d161454888640e916872dd2 . "./code.cg2js/cg-header-control.fasl")
(#xabee3506af1ffd56ee330a22ac233aa7 . "./code.cg2js/cg-hotspot.fasl")
(#x69f4b2327d5cb4aa222c33e685f67a34 . "./code.cg2js/cg-html-dialog.fasl")
(#x37980ead62cc79ea64eb8b91853c76e0 . "./code.cg2js/cg-html-widget.fasl")
(#x79423e8d76f20aa747cf46c9321e40f5 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xb1063e063161a71ecce2d314ffa2048e . "./code.cg2js/cg-icon.fasl")
(#x4e05f46f9144e2d8b52e5650818ebbbc . "./code.cg2js/cg-ie.fasl")
(#xf6efd7223dd7b72e8fd376a5c013f81b . "./code.cg2js/cg-intersect-posbox.fasl")
(#xc5706168249d0202a6218e9b20c6be29 . "./code.cg2js/cg-intersect.fasl")
(#x75944beffc12210e6afb4fba2aa4dd6f . "./code.cg2js/cg-item-list.fasl")
(#xd31a7c686c840e40c7adf63ef6d4ecbe . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xf6705ebda19ee665e869ea242de15353 . "./code.cg2js/cg-lamp.fasl")
(#xb6d3eb6b51baddc1ba2e5259fe4a5f97 . "./code.cg2js/cg-layout.fasl")
(#xcf859c81c2dd8f8e7b0f031b543e74f8 . "./code.cg2js/cg-lettered-menu.fasl")
(#xdd31e6b030967f25e5ba6b9058cb1748 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xff66eea0d24b12b82fce047067433576 . "./code.cg2js/cg-lisp-text.fasl")
(#x701511ec84d9e1c0a27c2dc572922dfd . "./code.cg2js/cg-lisp-widget.fasl")
(#xe70b9a2ef0ea5f9a30cd953263f292e9 . "./code.cg2js/cg-list-view.fasl")
(#xbbb827a388bc1ae9f4850df996fc762d . "./code.cg2js/cg-mci.fasl")
(#xc6e6ed39e4e3654930d6264e93dafcfd . "./code.cg2js/cg-menu-tooltip.fasl")
(#x19d8f6ba3e23b1f7c835fb84e44363c4 . "./code.cg2js/cg-menu.fasl")
(#x04a77dbaa9f78dccea917d94af87cacc . "./code.cg2js/cg-message-dialog.fasl")
(#xe6ff3fa4025f4f9108e470a9b4a68df8 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x95dbb8faba457a7a971289c1e3e521ac . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x963d4ce96c5cf1e6ccee3a8dae54918f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xcf55bcd68fa16a664cdd7ceefe659b16 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4f3be333df37404ef465031ffc3bb5b8 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xafdc1eb912d1e6f33c1d6327a3cc5586 . "./code.cg2js/cg-nodes.fasl")
(#x8f207c07960df69b8946b2b88507fc9f . "./code.cg2js/cg-object-editor-layout.fasl")
(#xdad6576972f5fb455f888c47e22c9f88 . "./code.cg2js/cg-object-editor.fasl")
(#xe8ea92ed354ee18979d19ca65d0791af . "./code.cg2js/cg-ocx.fasl")
(#xa49cec01df6f7feab7f3ba345896e36b . "./code.cg2js/cg-os-widget.fasl")
(#x3f33582bf430f1396860d3b2faeb53ed . "./code.cg2js/cg-os-window.fasl")
(#xf1ab8618f58634237041a84c80ec680b . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x26cf932850da92bceb3148f7eccf8847 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x2f4d597b67ce892d39828291a5b381c1 . "./code.cg2js/cg-outline.fasl")
(#x635427a204178300b9cad71c652364c3 . "./code.cg2js/cg-palette.fasl")
(#x950e9d18337a4b3ec5d2d9251c2c6ed0 . "./code.cg2js/cg-paren-matching.fasl")
(#x6453a79c8a9da3c584e705bfe14db82a . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x5474bda2a4527c65d75d8dda06786c0e . "./code.cg2js/cg-picture-widget.fasl")
(#xf08f6f83efc16cec0187b1e2a6f05ebe . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x82d033266c0108a0aa06eb205b785410 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x79a2189d6d794437e0427e03d12c3152 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x13d2a18e5ea4f3a4f1b26acb607b7bec . "./code.cg2js/cg-pixmap-widget.fasl")
(#xd85a21c8dcba8d1f26627a6ac837061f . "./code.cg2js/cg-pixmap.fasl")
(#xc644cc773adc6a42bff35ff5b8f9142a . "./code.cg2js/cg-player.fasl")
(#x26730e14e17ca2c973ce1a24b941cd2e . "./code.cg2js/cg-plot-widget.fasl")
(#xd0c9d87a7d1aab547a7f08724687f1e2 . "./code.cg2js/cg-printing.fasl")
(#xd9e7175cd8bcc03eb4b793a5b702be7a . "./code.cg2js/cg-progress-indicator.fasl")
(#x9bfa761c4a6164d1aa80c5515cf58aa4 . "./code.cg2js/cg-project-window.fasl")
(#x099ac9d65ea9b3be350953f999bb23f8 . "./code.cg2js/cg-property.fasl")
(#xf175f1eb4c1a6dc49c8a77f50c19dc91 . "./code.cg2js/cg-radio-button.fasl")
(#x2d03f20d9e1a87fd35c195c8ceed66bf . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x39bc0147954414687f38eef6a4b490fe . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xe11577be3301913d112fc1b0b5de4f4b . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x0f6a01b77b7a6a013f2bb064f2d0fcd1 . "./code.cg2js/cg-rich-edit.fasl")
(#xa34c379d181cadccff8f298eff475a1f . "./code.cg2js/cg-sample-file-menu.fasl")
(#x2cdbb565ac37271b41051d48f7640ce3 . "./code.cg2js/cg-scaling-stream.fasl")
(#xb0cd464dd39ab03df8af759e43b7426b . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xd128a09dc70848c80b0130f74ca3a74b . "./code.cg2js/cg-scroll-bar.fasl")
(#x618b45278951ff66762b958f224378f2 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xd1361d10f76130301420848da60d20ac . "./code.cg2js/cg-selected-object.fasl")
(#xb0a18320418f81adb9844bb83de1e187 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x2671b50302f0d82a6e7c5e4faeff65d9 . "./code.cg2js/cg-split-bar.fasl")
(#x0e110abb2c5cfc98a2f6b8721660d9a1 . "./code.cg2js/cg-static-text.fasl")
(#xb25626f138d341af17583c3d34bb34f6 . "./code.cg2js/cg-status-bar.fasl")
(#x54f28ccca201b10db3b527c8bce1b411 . "./code.cg2js/cg-string-dialog.fasl")
(#x3767593f2079c181a94e47d296572538 . "./code.cg2js/cg-tab-control.fasl")
(#xce5110493c802285f72a188005c65f43 . "./code.cg2js/cg-template-string.fasl")
(#x1ac917b251b62afb4921696acf9ea4b2 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x2fe4b188fb820334b28efbc184aae279 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xf777a83e79fea1b16d37c7f7a4fcd8d4 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xfe8a839bd1077c338dbdb4a148a0d395 . "./code.cg2js/cg-text-or-combo.fasl")
(#x01248c6803f93629771f527da7939865 . "./code.cg2js/cg-text-widget.fasl")
(#x87bc602d566acba2bfa12ab3cbc3bd31 . "./code.cg2js/cg-timer.fasl")
(#xa93c9ab6b71d205a359f840b4a776df1 . "./code.cg2js/cg-toggling-widget.fasl")
(#x417e5d690e139317fb9d5cadc97bd9ec . "./code.cg2js/cg-toolbar.fasl")
(#x730cfdb4368d0cde41ccc26dc323a7c5 . "./code.cg2js/cg-tooltip.fasl")
(#x8000e2da86c1638c2d903bd31c684376 . "./code.cg2js/cg-touch.fasl")
(#xd832bfc1d962454f441d324fbb1b1a0b . "./code.cg2js/cg-trackbar.fasl")
(#xd8415022b7103fa81505c5d0a00b66e6 . "./code.cg2js/cg-tray.fasl")
(#xafb1ad394953da57867e19fad2ad1859 . "./code.cg2js/cg-up-down-control.fasl")
(#x8f1311d11e225d4a8ff393e8107f70bd . "./code.cg2js/cg-utility-dialog.fasl")
(#xe61c16246b998621a3a296c6f6c1961a . "./code.cg2js/cg-web-browser-dde.fasl")
(#x3dcff988f6eac1a7dc167c59ee1e3474 . "./code.cg2js/cg-web-browser.fasl")
(#x195f264bf9299548f600af4006b030f9 . "./code.cg2js/cg-wrap-string.fasl")
(#xf4942470fd62f38dadb5824bde398a25 . "./code.cg2js/cg-yes-no-list.fasl")
(#x5bfed26ad9d8e7bae88f2f387dc314dc . "./code.cg2js/cg.fasl")
(#x258436ac9b88612e0a2e1366f9722483 . "./code.cg2js/cg-yes-no-string.fasl")
(#xfbf0987bd40d59e2472df08fdd1bcfb4 . "./code.idejs/ide-acache.fasl")
(#xe23f1a7ae7e3dabc5e7ee5b47a0bff07 . "./code.idejs/ide-apropos.fasl")
(#xd3058caacfb9d73700067a0424e85efe . "./code.idejs/ide-base.fasl")
(#x0c14051b82150e4026b3e62fd50a9e03 . "./code.idejs/ide-class-browser.fasl")
(#x74b25c7cc497f3c975500ce7073233b5 . "./code.idejs/ide-class-grapher.fasl")
(#x5ac7f697c7214eca023de8a4e8063ca0 . "./code.idejs/ide-clipboard.fasl")
(#x81b2b6c5b8754fa766ba61ac09f94c55 . "./code.idejs/ide-dev.fasl")
(#xbeae309a9d77ea497aba1946230b7d00 . "./code.idejs/ide-editor.fasl")
(#x7d2345d14073164d46fa50c60a138467 . "./code.idejs/ide-find-definitions.fasl")
(#xdcea5f855eef1f7d6687b735e79ed6e4 . "./code.idejs/ide-find-in-files.fasl")
(#x69b7c16447c0a6c8471668af4adfa8e7 . "./code.idejs/ide-form.fasl")
(#x69ddd69f60d83b8bce41729950a1b448 . "./code.idejs/ide-grapher.fasl")
(#xce9728fdf854162903faa0d3fb285b5c . "./code.idejs/ide-inspector.fasl")
(#x160a0c1db09791de2809f757ef8ded5d . "./code.idejs/ide-intro.fasl")
(#xbad634bbf6994decc80bbb6c8983a824 . "./code.idejs/ide-keyshort.fasl")
(#xcfb5da85fbb44b0fd9d16a1ae755c98b . "./code.idejs/ide-navigator.fasl")
(#x99fb631fcb1fad30a97c898d38de8cf9 . "./code.idejs/ide-package-list.fasl")
(#x4f7616a1549848c790831565dcb288cd . "./code.idejs/ide-patch-dialog.fasl")
(#x0d5bd739c3cc419967c51efe9bb13787 . "./code.idejs/ide-process-browser.fasl")
(#x78558fc54ceae43fd74aef472b12fc27 . "./code.idejs/ide-project.fasl")
(#xa0c571b1fa18dfb7f2619e3f23015d49 . "./code.idejs/ide-trace.fasl")
(#x573b6040fefcf47784ab5567512e1b03 . "./code.idejs/ide-window-list.fasl")
(#x8ba707d6ab7601511f297b582d89fa51 . "./code.idejs/ide.fasl")
(#x68149b79128815fb04933c4813dbe32f . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
